local retry = require 'retry'
local ran = {}
ran.RandomMessage = require 'ran'

math.randomseed(os.ts.time() % 1000)

-- We simulate intermittent errors - both fatal and non fatal.
local function UnreliableFunc(Data)
   if math.random(5) == 1 then
      error('We have a intermittent non fatal error!')
   end
end

-- The main function is the first function called from Iguana.
-- The Data argument will contain the message to be processed.
function main(Data)

   -- Simulating an unreliable function that does something
   local success, result = pcall(UnreliableFunc)

   -- Generate a random HL7 message
   local msg = ran.RandomMessage()
   local inMsg, msgType = hl7.parse{vmd = 'demo.vmd', data = msg}

   -- Randomly remove the MSH segment
   local outMsg = hl7.message{vmd='demo.vmd', name = msgType}
   outMsg:mapTree(inMsg)
   if math.random(5) == 1 then
      outMsg.MSH = nil
   end

   -- Return a successful or failed response
   if success then 
      net.http.respond{body=outMsg:S(), code=200}
   else
      net.http.respond{body=result, code=500}
   end

end